
// ImageData.h

#if !defined ImageData_h
#define ImageData_h

#include "glblcnst.h"
#include "RefCountPtr.h"
#include <vector>
#include "gdpixmap.h"
#include "scaler.h"
#include "gdfile.h"
#include "serialize.h"
#include "undogrid.h"


class TDocument;
class CLayer;
class image_block;


enum
{
	ClassID_ImageData_NoComp_1 = 1,
	ClassID_ImageData_NoComp_8 = 2
};


class
	ImageData:
public
	Serializable_Group
{
	// Stores image in memory.
	// May compress the image when required.
protected:
	Serializable_long width;
	Serializable_long height;
	Serializable_long rowbytes;
	bool changedSinceSave;
public:
	ImageData( long widthParam, long heightParam )
	{
		width = widthParam;
		height = heightParam;
	}
	virtual ~ImageData()
	{
	}

	std::vector< SerializableItem* > SerializableChildren()
	{
		std::vector< SerializableItem* > childrenArray;

		width.SetFieldID( 1 );
		childrenArray.push_back( &width );

		height.SetFieldID( 2 );
		childrenArray.push_back( &height );

		rowbytes.SetFieldID( 3 );
		childrenArray.push_back( &rowbytes );

		return childrenArray;
	}

	//virtual void				CopyImageToPixmap( const CzPixmap& dest_pmap ) = 0;
	//virtual void				CopyPixmapToImage( const CzPixmap& dest_pmap ) = 0;
	virtual void CopyImageToPixmap( const gd::Pixmap_1Bit& dest_pmap )
	{
		assert( false );
	}
	virtual void CopyPixmapToImage( const gd::Pixmap_1Bit& srcPmap )
	{
		assert( false );
	}

	virtual void CopyPixmapToImage( const gd::Pixmap_8Bit& srcPmap )
	{
		assert( false );
	}

	virtual gd::BWPixmapPtr EfficientImageToPixmap_1Bit( const gd::irect& imageRect )
	{
		assert( false );
		return gd::BWPixmapPtr( 0 );
	}
	virtual long		pixel_value( gd::ipoint the_pt )
	{
		assert(false);
		return 0;
	}
	long			Height() { return height.value; }
	long			Width() { return width.value; }
	bool			ChangedSinceSave() { return changedSinceSave; }
	void			SetHasBeenSaved() { changedSinceSave = false; }
	virtual void	UndoLastAction() { assert( false ); }
	virtual void	StartUndoableAction() { assert( false ); }
	virtual gd::GrayPixmapPtr RenderOn1BytePixmap( const gd::irect& pixmapBounds, Scaler scaler )
	{
		assert( false );
		return gd::GrayPixmapPtr( 0 );
	}
	virtual void AddImage( const gd::Pixmap_1Bit& imagePixmap, long shade ) { assert( false ); }
	//virtual void AddQResImage( const gd::Pixmap_1Bit& imagePixmap, long shade ) { assert( false ); }
	virtual void ApplyImage( const gd::Pixmap_8Bit& imagePixmap, long shade ) { assert( false ); }
	virtual void ApplyBlend( const gd::Pixmap_8Bit& blendPixmap, const gd::Pixmap_8Bit& brushMask, long shade, long shade2 ) { assert( false ); }
	virtual long Pixel( const gd::ipoint& position ) const { assert( false ); return 0; }
	virtual void SetPixel( const gd::ipoint& position, long value ) const { assert( false ); }
	virtual void RenderOn1BytePixmap( gd::GrayPixmapPtr pixmap, Scaler scaler ) { assert( false ); }
	virtual void RenderOn1BitPixmap( gd::BWPixmapPtr pixmap, Scaler scaler ) { assert( false ); }	
	virtual void SaveDataToFile( const gd::GDFile& saveFile ) { assert( false ); }
	virtual void InitializeUndoGrid() { assert( false ); }
	virtual char* DataPtr() = 0;
	virtual long DataSize() = 0;
};


class
	ImageData_NoComp_1 :
public
	ImageData
{
	//  bit image with no compression abilities
private:
	SerializablePointer< gd::BWPixmapPtr > pixmap;
	//RefCountPtr< UndoGrid >	undoGrid;
	UndoGrid_b< gd::Pixmap_1Bit > undoGrid;
public:
	ImageData_NoComp_1( long width, long height );
	ImageData_NoComp_1( gd::BWPixmapPtr& layerPixmap );
	~ImageData_NoComp_1();

	std::vector< SerializableItem* > SerializableChildren()
	{
		std::vector< SerializableItem* > childrenArray = ImageData::SerializableChildren();

		pixmap.SetFieldID( 4 );
		childrenArray.push_back( &pixmap );

		return childrenArray;
	}

	void		CopyImageToPixmap( const gd::Pixmap_1Bit& dest_pmap );
	void		CopyPixmapToImage( const gd::Pixmap_1Bit& srcPmap );
	gd::BWPixmapPtr	EfficientImageToPixmap_1Bit( const gd::irect& imageRect );
	void		UndoLastAction();
	void		StartUndoableAction();
	long		Pixel( const gd::ipoint& position ) const; 
	void		SetPixel( const gd::ipoint& position, long value ) const;
	void		RenderOn1BytePixmap( gd::GrayPixmapPtr pixmap, Scaler scaler );
	void		RenderOn1BitPixmap( gd::BWPixmapPtr pixmap, Scaler scaler );
	void		SaveDataToFile( const gd::GDFile& saveFile );
	long		ClassHeirarchyMemberID() { return ClassID_ImageData_NoComp_1; }
	void		InitializeUndoGrid();
	char*		DataPtr() { return ( char* ) pixmap.value->data.Data(); }
	long		DataSize()  { return pixmap.value->data.RawDataSize(); }
};


class
	ImageData_NoComp_8 :
public
	ImageData
{
	//  bit image with no compression abilities
private:
	SerializablePointer< gd::GrayPixmapPtr > pixmap;
	//gd::GrayPixmapPtr pixmap;
	//RefCountPtr< UndoGrid >	undoGrid;
	UndoGrid_b< gd::Pixmap_8Bit > undoGrid;
public:
	ImageData_NoComp_8( long width, long height );
	~ImageData_NoComp_8();

	std::vector< SerializableItem* > SerializableChildren()
	{
		std::vector< SerializableItem* > childrenArray = ImageData::SerializableChildren();

		pixmap.SetFieldID( 5 );
		childrenArray.push_back( &pixmap );

		return childrenArray;
	}

	//void		CopyImageToPixmap( const gd::Pixmap_1Bit& dest_pmap );
	//void		CopyPixmapToImage( const gd::Pixmap_1Bit& srcPmap );
	//BWPixmapPtr	EfficientImageToPixmap_1Bit( const gd::irect& imageRect );
	void		UndoLastAction();
	void		StartUndoableAction();
	gd::GrayPixmapPtr RenderOn1BytePixmap( const gd::irect& pixmapBounds, Scaler scaler );
	void		AddImage( const gd::Pixmap_1Bit& imagePixmap, long shade );
	void		ApplyImage( const gd::Pixmap_8Bit& imagePixmap, long shade );
	void		ApplyBlend( const gd::Pixmap_8Bit& blendPixmap, const gd::Pixmap_8Bit& brushMask, long shade, long shade2);
	long		Pixel( const gd::ipoint& position ) const;
	void		SetPixel( const gd::ipoint& position, long value ) const;
	void		RenderOn1BytePixmap( gd::GrayPixmapPtr pixmap, Scaler scaler );
	void		SaveDataToFile( const gd::GDFile& saveFile );
	long		ClassHeirarchyMemberID() { return ClassID_ImageData_NoComp_8; }
	void		InitializeUndoGrid();
	char*		DataPtr() { return ( char* ) pixmap.value->data.Data(); }
	long		DataSize() { return pixmap.value->data.RawDataSize(); }
	void		CopyPixmapToImage( const gd::Pixmap_8Bit& srcPmap );
};


class
	ImageData_CompBlocks_1 :
public
	ImageData
{
public:
	TDocument*		fdocument;
	CLayer*			flayer;
	image_block*	fcomp_mem;

	ImageData_CompBlocks_1(long width, long height, TDocument* pdoc, CLayer* player,
		create_temp_file create_temp);
	~ImageData_CompBlocks_1(void);

	//void CopyImageToPixmap(const CzPixmap& dest_pmap);
	//void CopyPixmapToImage( const CzPixmap& dest_pmap );

//	void copy_mybits_to_mem(const CzPixmap& dest_pmap);
	void copy_1_line_to_image(long pline_num, char* pdata);

//	void CopyImageToPixmap(const CzPixmap& ppmap);
//	void copy_pixmap_to_image(CzPixmap* ppmap);
//	void write_all_memory_to_disk(void);
//	void copy_q_image_to_full_pixmap(CzPixmap* ppmap);
//	void copy_full_pixmap_to_q_image(CzPixmap* ppmap);
//	long pixel_value(ipoint the_pt);
};


class
	ImageData_CompBlocks_8 :
public
	ImageData
{
public:
	TDocument*		fdocument;
	CLayer*			flayer;
	image_block*	fcomp_mem;

	ImageData_CompBlocks_8(long width, long height, TDocument* pdoc, CLayer* player,
		create_temp_file create_temp);
	~ImageData_CompBlocks_8();

	//void CopyImageToPixmap( const CzPixmap& ppmap );
	//void CopyPixmapToImage( const CzPixmap& ppmap );
	void write_all_memory_to_disk();
	//void copy_q_image_to_full_pixmap( CzPixmap& ppmap );
	//void copy_full_pixmap_to_q_image( CzPixmap& ppmap );
	virtual long pixel_value( gd::ipoint the_pt );
};


class
	ImageDataFactory
{
	// Chooses and returns appropriate form of CImageData
public:
	static ImageData* NewImageData( long bitsPerPixel, long width, long height )
	{
		switch ( bitsPerPixel )
		{
			case 1:
				return new ImageData_NoComp_1( width, height );
				break;
			case 8:
				return new ImageData_NoComp_8( width, height );
				break;
			default:	
				assert( false );
				return 0;
		}
	}
	static ImageData* NewImageData( gd::BWPixmapPtr& layerPixmap )
	{
		return new ImageData_NoComp_1( layerPixmap );
	}
};


typedef RefCountPtr< ImageData > ImageDataPtr;

//bool SerializablePointer< ImageDataPtr >::IsNull();
template <> bool SerializablePointer< ImageDataPtr >::IsNull();
//template <>
//bool SerializablePointer< ImageDataPtr >::IsNull();
//{
//	return value.IsNull();
//}


#endif

